# Generated by Plain 0.127.2 on 2026-03-25 16:53

from plain.postgres import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainjobs", "0008_alter_jobrequest_options"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="jobprocess",
            name="plainjobs_j_job_cla_19f3c1_idx",
        ),
        migrations.RemoveIndex(
            model_name="jobrequest",
            name="plainjobs_j_job_cla_a18abf_idx",
        ),
    ]



from __future__ import annotations

import datetime
import signal
from typing import Any

import click

from plain.cli import SettingOption, register_cli
from plain.logs import get_framework_logger
from plain.runtime import settings
from plain.utils import timezone

from .models import JobProcess, JobRequest, JobResult
from .registry import jobs_registry
from .scheduling import load_schedule
from .workers import Worker

logger = get_framework_logger()


@register_cli("jobs")
@click.group()
def cli() -> None:
    """Background job management"""


@cli.command()
@click.option(
    "queues",
    "--queue",
    default=["default"],
    multiple=True,
    type=str,
    help="Queue to process",
)
@click.option(
    "--max-processes",
    "max_processes",
    type=int,
    cls=SettingOption,
    setting="JOBS_WORKER_MAX_PROCESSES",
)
@click.option(
    "--max-jobs-per-process",
    "max_jobs_per_process",
    type=int,
    cls=SettingOption,
    setting="JOBS_WORKER_MAX_JOBS_PER_PROCESS",
)
@click.option(
    "--max-pending-per-process",
    "max_pending_per_process",
    type=int,
    cls=SettingOption,
    setting="JOBS_WORKER_MAX_PENDING_PER_PROCESS",
)
@click.option(
    "--stats-every",
    "stats_every",
    type=int,
    cls=SettingOption,
    setting="JOBS_WORKER_STATS_EVERY",
)
@click.option(
    "--reload",
    is_flag=True,
    help="Watch files and auto-reload worker on changes",
)
def worker(
    queues: tuple[str, ...],
    max_processes: int | None,
    max_jobs_per_process: int | None,
    max_pending_per_process: int,
    stats_every: int,
    reload: bool,
) -> None:
    """Run the job worker"""
    jobs_schedule = load_schedule(settings.JOBS_SCHEDULE)

    worker_kwargs = {
        "queues": list(queues),
        "jobs_schedule": jobs_schedule,
        "max_processes": max_processes,
        "max_jobs_per_process": max_jobs_per_process,
        "max_pending_per_process": max_pending_per_process,
        "stats_every": stats_every,
    }

    if reload:
        _run_with_reload(worker_kwargs)
    else:
        _run_once(worker_kwargs)


def _run_with_reload(worker_kwargs: dict[str, Any]) -> None:
    from plain.internal.reloader import Reloader

    should_restart = {"value": True}
    current_worker: dict[str, Worker | None] = {"instance": None}

    def file_changed(filename: str) -> None:
        if current_worker["instance"]:
            current_worker["instance"].shutdown()

    def signal_shutdown(signalnum: int, _: Any) -> None:
        should_restart["value"] = False
        if current_worker["instance"]:
            current_worker["instance"].shutdown()

    signal.signal(signal.SIGTERM, signal_shutdown)
    signal.signal(signal.SIGINT, signal_shutdown)

    reloader = Reloader(callback=file_changed, watch_html=False)
    reloader.start()

    while should_restart["value"]:
        w = Worker(**worker_kwargs)
        current_worker["instance"] = w
        w.run()


def _run_once(worker_kwargs: dict[str, Any]) -> None:
    w = Worker(**worker_kwargs)

    def _shutdown(signalnum: int, _: Any) -> None:
        logger.info(
            "Job worker shutdown signal received",
            extra={"signalnum": signalnum},
        )
        w.shutdown()

    signal.signal(signal.SIGTERM, _shutdown)
    signal.signal(signal.SIGINT, _shutdown)

    w.run()


@cli.command()
def clear() -> None:
    """Clear completed job results"""
    cutoff = timezone.now() - datetime.timedelta(
        seconds=settings.JOBS_RESULTS_RETENTION
    )
    click.echo(f"Clearing job results created before {cutoff}")
    results = JobResult.query.filter(created_at__lt=cutoff).delete()
    click.echo(f"Deleted {results[0]} jobs")


@cli.command()
def stats() -> None:
    """Show job queue statistics"""
    pending = JobRequest.query.count()
    processing = JobProcess.query.count()

    successful = JobResult.query.successful().count()
    errored = JobResult.query.errored().count()
    lost = JobResult.query.lost().count()

    click.secho(f"Pending: {pending}", bold=True)
    click.secho(f"Processing: {processing}", bold=True)
    click.secho(f"Successful: {successful}", bold=True, fg="green")
    click.secho(f"Errored: {errored}", bold=True, fg="red")
    click.secho(f"Lost: {lost}", bold=True, fg="yellow")


@cli.command()
def purge() -> None:
    """Delete all pending and running jobs"""
    if not click.confirm(
        "Are you sure you want to clear all running and pending jobs? This will delete all current Jobs and JobRequests"
    ):
        return

    deleted = JobRequest.query.all().delete()[0]
    click.echo(f"Deleted {deleted} job requests")

    deleted = JobProcess.query.all().delete()[0]
    click.echo(f"Deleted {deleted} jobs")


@cli.command()
@click.argument("job_class_name", type=str)
def run(job_class_name: str) -> None:
    """Run a job directly without a worker"""
    job = jobs_registry.load_job(job_class_name, {"args": [], "kwargs": {}})
    click.secho("Loaded job: ", bold=True, nl=False)
    print(job)
    job.run()


@cli.command("list")
def list_jobs() -> None:
    """List all registered jobs"""
    for name, job_class in jobs_registry.jobs.items():
        click.secho(name, bold=True, nl=False)
        description = job_class.__doc__.strip() if job_class.__doc__ else ""
        if description:
            click.secho(f": {description}", dim=True)
        else:
            click.echo("")



# Generated by Plain 0.63.0 on 2025-09-15 21:29

from plain import postgres
from plain.postgres import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainjobs", "0002_job_span_id_job_trace_id_jobrequest_span_id_and_more"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="Job",
            new_name="JobProcess",
        ),
        migrations.RemoveIndex(
            model_name="jobresult",
            name="plainworker_job_uui_8307d1_idx",
        ),
        migrations.RenameField(
            model_name="jobresult",
            old_name="job_uuid",
            new_name="job_process_uuid",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainworker_created_0d3928_idx",
            old_name="plainworker_created_a02317_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainworker_queue_2550ba_idx",
            old_name="plainworker_queue_077806_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainworker_unique__9dc0bb_idx",
            old_name="plainworker_unique__04d87b_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainworker_started_b80ec5_idx",
            old_name="plainworker_started_143df5_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainworker_job_cla_fe2b70_idx",
            old_name="plainworker_job_cla_884b46_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainworker_job_req_357898_idx",
            old_name="plainworker_job_req_db2681_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainworker_trace_i_da2cfa_idx",
            old_name="plainworker_trace_i_d2b645_idx",
        ),
        migrations.AddIndex(
            model_name="jobresult",
            index=postgres.Index(
                fields=["job_process_uuid"], name="plainworker_job_pro_ceabfb_idx"
            ),
        ),
        # To fix subsequent migrations...
        migrations.AlterModelTable(
            name="JobRequest",
            table="plainworker_jobrequest",
        ),
        migrations.AlterModelTable(
            name="JobProcess",
            table="plainworker_jobprocess",
        ),
        migrations.AlterModelTable(
            name="JobResult",
            table="plainworker_jobresult",
        ),
    ]



# Generated by Plain 0.52.2 on 2025-07-17 19:33

from plain import postgres
from plain.postgres import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainjobs", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="job",
            name="span_id",
            field=postgres.CharField(allow_null=True, max_length=18, required=False),
        ),
        migrations.AddField(
            model_name="job",
            name="trace_id",
            field=postgres.CharField(allow_null=True, max_length=34, required=False),
        ),
        migrations.AddField(
            model_name="jobrequest",
            name="span_id",
            field=postgres.CharField(allow_null=True, max_length=18, required=False),
        ),
        migrations.AddField(
            model_name="jobrequest",
            name="trace_id",
            field=postgres.CharField(allow_null=True, max_length=34, required=False),
        ),
        migrations.AddField(
            model_name="jobresult",
            name="span_id",
            field=postgres.CharField(allow_null=True, max_length=18, required=False),
        ),
        migrations.AddField(
            model_name="jobresult",
            name="trace_id",
            field=postgres.CharField(allow_null=True, max_length=34, required=False),
        ),
        migrations.AddIndex(
            model_name="job",
            index=postgres.Index(
                fields=["trace_id"], name="plainworker_trace_i_d2b645_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobrequest",
            index=postgres.Index(
                fields=["trace_id"], name="plainworker_trace_i_e9dfc5_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobresult",
            index=postgres.Index(
                fields=["trace_id"], name="plainworker_trace_i_00c75f_idx"
            ),
        ),
    ]



# Step 2: Rename constraints and indexes from plainworker_* to plainjobs_*
# (Tables were renamed to plainjobs_* in migration 0004)

from plain import postgres
from plain.postgres import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainjobs", "0004_rename_tables_to_plainjobs"),
    ]

    operations = [
        # Remove old constraints (on plainjobs_* tables now)
        migrations.RemoveConstraint(
            model_name="jobprocess",
            name="plainworker_job_unique_uuid",
        ),
        migrations.RemoveConstraint(
            model_name="jobrequest",
            name="plainworker_jobrequest_unique_job_class_key",
        ),
        migrations.RemoveConstraint(
            model_name="jobrequest",
            name="plainworker_jobrequest_unique_uuid",
        ),
        migrations.RemoveConstraint(
            model_name="jobresult",
            name="plainworker_jobresult_unique_uuid",
        ),
        # Rename indexes
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_j_created_04fbb8_idx",
            old_name="plainworker_created_0d3928_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_j_queue_d07d21_idx",
            old_name="plainworker_queue_2550ba_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_j_unique__67172c_idx",
            old_name="plainworker_unique__9dc0bb_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_j_started_5cd62a_idx",
            old_name="plainworker_started_b80ec5_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_j_job_cla_19f3c1_idx",
            old_name="plainworker_job_cla_fe2b70_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_j_job_req_32b6eb_idx",
            old_name="plainworker_job_req_357898_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_j_trace_i_9f93c8_idx",
            old_name="plainworker_trace_i_da2cfa_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_j_priorit_fd4fac_idx",
            old_name="plainworker_priorit_785e73_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_j_created_1eeb20_idx",
            old_name="plainworker_created_c81fe5_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_j_queue_b34b5a_idx",
            old_name="plainworker_queue_2614aa_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_j_start_a_f3b8da_idx",
            old_name="plainworker_start_a_4d6020_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_j_unique__42f6a6_idx",
            old_name="plainworker_unique__21a534_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_j_job_cla_a18abf_idx",
            old_name="plainworker_job_cla_3e7dea_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_j_trace_i_194003_idx",
            old_name="plainworker_trace_i_e9dfc5_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_j_created_7978bf_idx",
            old_name="plainworker_created_6894c5_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_j_job_pro_751a64_idx",
            old_name="plainworker_job_pro_ceabfb_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_j_started_6fb2ce_idx",
            old_name="plainworker_started_9bce76_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_j_ended_a_648f25_idx",
            old_name="plainworker_ended_a_63caaf_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_j_status_1ef683_idx",
            old_name="plainworker_status_a7ca35_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_j_job_req_3ddecf_idx",
            old_name="plainworker_job_req_1e1bf2_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_j_job_cla_8791b4_idx",
            old_name="plainworker_job_cla_d138b5_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_j_queue_0a2178_idx",
            old_name="plainworker_queue_23d8fe_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_j_trace_i_02f370_idx",
            old_name="plainworker_trace_i_00c75f_idx",
        ),
        # Add new constraints (on plainworker_* tables, but with new names)
        migrations.AddConstraint(
            model_name="jobprocess",
            constraint=postgres.UniqueConstraint(
                fields=("uuid",), name="plainjobs_job_unique_uuid"
            ),
        ),
        migrations.AddConstraint(
            model_name="jobrequest",
            constraint=postgres.UniqueConstraint(
                condition=postgres.Q(("retry_attempt", 0), ("unique_key__gt", "")),
                fields=("job_class", "unique_key"),
                name="plainjobs_jobrequest_unique_job_class_key",
            ),
        ),
        migrations.AddConstraint(
            model_name="jobrequest",
            constraint=postgres.UniqueConstraint(
                fields=("uuid",), name="plainjobs_jobrequest_unique_uuid"
            ),
        ),
        migrations.AddConstraint(
            model_name="jobresult",
            constraint=postgres.UniqueConstraint(
                fields=("uuid",), name="plainjobs_jobresult_unique_uuid"
            ),
        ),
    ]



# Generated by Plain 0.127.2 on 2026-03-25 20:19

from plain.postgres import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("plainjobs", "0009_remove_jobprocess_plainjobs_j_job_cla_19f3c1_idx_and_more"),
    ]

    operations = [
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_jobprocess_created_at_idx",
            old_name="plainjobs_j_created_04fbb8_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_jobprocess_queue_idx",
            old_name="plainjobs_j_queue_d07d21_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_jobprocess_concurrency_key_idx",
            old_name="plainjobs_j_concurr_ad4464_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_jobprocess_started_at_idx",
            old_name="plainjobs_j_started_5cd62a_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_jobprocess_job_request_uuid_idx",
            old_name="plainjobs_j_job_req_32b6eb_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_jobprocess_trace_id_idx",
            old_name="plainjobs_j_trace_i_9f93c8_idx",
        ),
        migrations.RenameIndex(
            model_name="jobprocess",
            new_name="plainjobs_jobprocess_uuid_idx",
            old_name="plainjobs_j_uuid_cd8cd3_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_jobrequest_priority_idx",
            old_name="plainjobs_j_priorit_fd4fac_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_jobrequest_created_at_idx",
            old_name="plainjobs_j_created_1eeb20_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_jobrequest_queue_idx",
            old_name="plainjobs_j_queue_b34b5a_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_jobrequest_start_at_idx",
            old_name="plainjobs_j_start_a_f3b8da_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_jobrequest_concurrency_key_idx",
            old_name="plainjobs_j_concurr_c10926_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_jobrequest_trace_id_idx",
            old_name="plainjobs_j_trace_i_194003_idx",
        ),
        migrations.RenameIndex(
            model_name="jobrequest",
            new_name="plainjobs_jobrequest_uuid_idx",
            old_name="plainjobs_j_uuid_c41c85_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_jobresult_created_at_idx",
            old_name="plainjobs_j_created_7978bf_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_jobresult_job_process_uuid_idx",
            old_name="plainjobs_j_job_pro_751a64_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_jobresult_started_at_idx",
            old_name="plainjobs_j_started_6fb2ce_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_jobresult_ended_at_idx",
            old_name="plainjobs_j_ended_a_648f25_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_jobresult_status_idx",
            old_name="plainjobs_j_status_1ef683_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_jobresult_job_request_uuid_idx",
            old_name="plainjobs_j_job_req_3ddecf_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_jobresult_job_class_idx",
            old_name="plainjobs_j_job_cla_8791b4_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_jobresult_queue_idx",
            old_name="plainjobs_j_queue_0a2178_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_jobresult_trace_id_idx",
            old_name="plainjobs_j_trace_i_02f370_idx",
        ),
        migrations.RenameIndex(
            model_name="jobresult",
            new_name="plainjobs_jobresult_uuid_idx",
            old_name="plainjobs_j_uuid_b33b4c_idx",
        ),
    ]



# Generated by Plain 0.52.2 on 2025-07-08 01:17

import uuid

from plain import postgres
from plain.postgres import migrations


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Job",
            fields=[
                ("id", postgres.PrimaryKeyField()),
                ("uuid", postgres.UUIDField(default=uuid.uuid4)),
                ("created_at", postgres.DateTimeField(auto_now_add=True)),
                ("started_at", postgres.DateTimeField(allow_null=True, required=False)),
                ("job_request_uuid", postgres.UUIDField()),
                ("job_class", postgres.CharField(max_length=255)),
                ("parameters", postgres.JSONField(allow_null=True, required=False)),
                ("priority", postgres.IntegerField(default=0)),
                ("source", postgres.TextField(required=False)),
                ("queue", postgres.CharField(default="default", max_length=255)),
                ("retries", postgres.IntegerField(default=0)),
                ("retry_attempt", postgres.IntegerField(default=0)),
                ("unique_key", postgres.CharField(max_length=255, required=False)),
            ],
            options={
                "ordering": ["-created_at"],
            },
        ),
        migrations.CreateModel(
            name="JobRequest",
            fields=[
                ("id", postgres.PrimaryKeyField()),
                ("created_at", postgres.DateTimeField(auto_now_add=True)),
                ("uuid", postgres.UUIDField(default=uuid.uuid4)),
                ("job_class", postgres.CharField(max_length=255)),
                ("parameters", postgres.JSONField(allow_null=True, required=False)),
                ("priority", postgres.IntegerField(default=0)),
                ("source", postgres.TextField(required=False)),
                ("queue", postgres.CharField(default="default", max_length=255)),
                ("retries", postgres.IntegerField(default=0)),
                ("retry_attempt", postgres.IntegerField(default=0)),
                ("unique_key", postgres.CharField(max_length=255, required=False)),
                ("start_at", postgres.DateTimeField(allow_null=True, required=False)),
            ],
            options={
                "ordering": ["priority", "-created_at"],
            },
        ),
        migrations.CreateModel(
            name="JobResult",
            fields=[
                ("id", postgres.PrimaryKeyField()),
                ("uuid", postgres.UUIDField(default=uuid.uuid4)),
                ("created_at", postgres.DateTimeField(auto_now_add=True)),
                ("job_uuid", postgres.UUIDField()),
                ("started_at", postgres.DateTimeField(allow_null=True, required=False)),
                ("ended_at", postgres.DateTimeField(allow_null=True, required=False)),
                ("error", postgres.TextField(required=False)),
                (
                    "status",
                    postgres.CharField(
                        choices=[
                            ("SUCCESSFUL", "Successful"),
                            ("ERRORED", "Errored"),
                            ("CANCELLED", "Cancelled"),
                            ("LOST", "Lost"),
                        ],
                        max_length=20,
                    ),
                ),
                ("job_request_uuid", postgres.UUIDField()),
                ("job_class", postgres.CharField(max_length=255)),
                ("parameters", postgres.JSONField(allow_null=True, required=False)),
                ("priority", postgres.IntegerField(default=0)),
                ("source", postgres.TextField(required=False)),
                ("queue", postgres.CharField(default="default", max_length=255)),
                ("retries", postgres.IntegerField(default=0)),
                ("retry_attempt", postgres.IntegerField(default=0)),
                ("unique_key", postgres.CharField(max_length=255, required=False)),
                (
                    "retry_job_request_uuid",
                    postgres.UUIDField(allow_null=True, required=False),
                ),
            ],
            options={
                "ordering": ["-created_at"],
            },
        ),
        migrations.AddIndex(
            model_name="job",
            index=postgres.Index(
                fields=["created_at"], name="plainworker_created_a02317_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="job",
            index=postgres.Index(fields=["queue"], name="plainworker_queue_077806_idx"),
        ),
        migrations.AddIndex(
            model_name="job",
            index=postgres.Index(
                fields=["unique_key"], name="plainworker_unique__04d87b_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="job",
            index=postgres.Index(
                fields=["started_at"], name="plainworker_started_143df5_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="job",
            index=postgres.Index(
                fields=["job_class"], name="plainworker_job_cla_884b46_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="job",
            index=postgres.Index(
                fields=["job_request_uuid"], name="plainworker_job_req_db2681_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="job",
            index=postgres.Index(
                fields=["job_class", "unique_key"], name="job_class_unique_key"
            ),
        ),
        migrations.AddConstraint(
            model_name="job",
            constraint=postgres.UniqueConstraint(
                fields=("uuid",), name="plainworker_job_unique_uuid"
            ),
        ),
        migrations.AddIndex(
            model_name="jobrequest",
            index=postgres.Index(
                fields=["priority"], name="plainworker_priorit_785e73_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobrequest",
            index=postgres.Index(
                fields=["created_at"], name="plainworker_created_c81fe5_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobrequest",
            index=postgres.Index(fields=["queue"], name="plainworker_queue_2614aa_idx"),
        ),
        migrations.AddIndex(
            model_name="jobrequest",
            index=postgres.Index(
                fields=["start_at"], name="plainworker_start_a_4d6020_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobrequest",
            index=postgres.Index(
                fields=["unique_key"], name="plainworker_unique__21a534_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobrequest",
            index=postgres.Index(
                fields=["job_class"], name="plainworker_job_cla_3e7dea_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobrequest",
            index=postgres.Index(
                fields=["job_class", "unique_key"], name="job_request_class_unique_key"
            ),
        ),
        migrations.AddConstraint(
            model_name="jobrequest",
            constraint=postgres.UniqueConstraint(
                condition=postgres.Q(("retry_attempt", 0), ("unique_key__gt", "")),
                fields=("job_class", "unique_key"),
                name="plainworker_jobrequest_unique_job_class_key",
            ),
        ),
        migrations.AddConstraint(
            model_name="jobrequest",
            constraint=postgres.UniqueConstraint(
                fields=("uuid",), name="plainworker_jobrequest_unique_uuid"
            ),
        ),
        migrations.AddIndex(
            model_name="jobresult",
            index=postgres.Index(
                fields=["created_at"], name="plainworker_created_6894c5_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobresult",
            index=postgres.Index(
                fields=["job_uuid"], name="plainworker_job_uui_8307d1_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobresult",
            index=postgres.Index(
                fields=["started_at"], name="plainworker_started_9bce76_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobresult",
            index=postgres.Index(
                fields=["ended_at"], name="plainworker_ended_a_63caaf_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobresult",
            index=postgres.Index(
                fields=["status"], name="plainworker_status_a7ca35_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobresult",
            index=postgres.Index(
                fields=["job_request_uuid"], name="plainworker_job_req_1e1bf2_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobresult",
            index=postgres.Index(
                fields=["job_class"], name="plainworker_job_cla_d138b5_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="jobresult",
            index=postgres.Index(fields=["queue"], name="plainworker_queue_23d8fe_idx"),
        ),
        migrations.AddConstraint(
            model_name="jobresult",
            constraint=postgres.UniqueConstraint(
                fields=("uuid",), name="plainworker_jobresult_unique_uuid"
            ),
        ),
    ]
